package middleware

import (
	"context"
	"net/http"
)

// StatusWriter is a http.ResponseWriter wrapper to save the status and length of the response
type StatusWriter struct {
	http.ResponseWriter
	Status int
	Length int
}

// WriteHeader wraps the response writer call and stores the status arg
func (w *StatusWriter) WriteHeader(status int) {
	w.Status = status
	w.ResponseWriter.WriteHeader(status)
}

// Write wraps the response writer call and stores the status and content length
func (w *StatusWriter) Write(b []byte) (int, error) {
	if w.Status == 0 {
		w.Status = 200
	}
	n, err := w.ResponseWriter.Write(b)
	w.Length += n
	return n, err
}

func (mwf RouteMiddlewareFactory) WriteStatus(next http.Handler) http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		sw := &StatusWriter{ResponseWriter: w}
		defer func() {
			*r = *r.Clone(context.WithValue(r.Context(), Status, sw.Status))
		}()
		next.ServeHTTP(sw, r)
	}
}
