// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package middleware

import (
	"context"
	"net/http"
	"time"
)

const RetryAfterHeaderKey = "Retry-After"

// TimeoutMiddleware -- function to handle timeouts
//
//goland:noinspection GoMixedReceiverTypes
func (mwf RouteMiddlewareFactory) TimeoutMiddleware(handler http.Handler, timeoutDuration time.Duration) http.Handler {
	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
		// Create a place to save the gin context key-value pairs which would otherwise be canclled with the timeout context
		*r = *r.Clone(context.WithValue(r.Context(), SavedGinContext, &SavedContext{}))
		http.TimeoutHandler(handler, timeoutDuration, "timeout hit").ServeHTTP(w, r)
	})
}
