package redisclient

import (
	"context"
	"time"

	"gitlab.bit9.local/psc/ems/hinting/internal/config"
)

type redisClientWithTimeout struct {
	RedisClient
	timeout time.Duration
}

func NewRedisClientWithTimeoutFromConfig(cfg *config.Config) (client *redisClientWithTimeout, err error) {
	c, err := getRedisClientAndPing(cfg)
	if err != nil {
		return
	}

	return &redisClientWithTimeout{RedisClient: c, timeout: cfg.NoSQLTimeout}, nil
}

func NewRedisClientWithTimeoutFromRedisClient(redisClient RedisClient, timeout time.Duration) (client *redisClientWithTimeout) {
	return &redisClientWithTimeout{RedisClient: redisClient, timeout: timeout}
}

func (client *redisClientWithTimeout) Get(ctx context.Context, cacheKey string) ([]byte, error) {
	timedCtx, cancelFunc := context.WithTimeout(ctx, client.timeout)
	defer cancelFunc()

	return client.RedisClient.Get(timedCtx, cacheKey)
}

func (client *redisClientWithTimeout) Set(ctx context.Context, cacheKey string, data []byte) error {
	timedCtx, cancelFunc := context.WithTimeout(ctx, client.timeout)
	defer cancelFunc()

	return client.RedisClient.Set(timedCtx, cacheKey, data)
}

func (client *redisClientWithTimeout) Del(ctx context.Context, cacheKey string) error {
	timedCtx, cancelFunc := context.WithTimeout(ctx, client.timeout)
	defer cancelFunc()

	return client.RedisClient.Del(timedCtx, cacheKey)
}

func (client *redisClientWithTimeout) Ping(ctx context.Context) error {
	return client.RedisClient.Ping(ctx)
}

func (client *redisClientWithTimeout) Close() error {
	return client.RedisClient.Close()
}
